<?php
/**
 * The model file of ssoauth module of ZenTaoPMS.
 *
 * @copyright   Copyright 2009-2020 禅道软件（青岛）有限公司(ZenTao Software (Qingdao) Co., Ltd. www.cnezsoft.com)
 * @license     ZPL(http://zpl.pub/page/zplv12.html) or AGPL(https://www.gnu.org/licenses/agpl-3.0.en.html)
 * @package     oidc
 * @link        http://www.zentao.net
 */
class oidcModel extends model
{
    public function __construct()
    {
        parent::__construct();
        $this->loadModel('requestlog');
    }

    /**
     * Meage system config to oidc config.
     *
     * @param mixed $config
     * @access public
     * @return void
     */
    public function meageConfig($config)
    {
        $systemConfig = $this->loadModel('setting')->getItems('owner=system&module=oidc');
        /* Get single sign-on configuration items. */
        foreach($systemConfig as $oidc)
        {
            $key = array_search($oidc->key, $config);
            if($key !== false)
            {
                $config[$key] = $oidc->value;
            }
        }
        return $config;
    }

    /**
     * Get oidc code.
     *
     * @access public
     * @return void
     */
    public function getOidcCode()
    {
        $this->app->loadConfig('oidc');
        $codeParams = $this->meageConfig($this->config->oidc->codeParams);
        // 生成用于登录的一次性地址，之后可以引导用户访问此地址
        $codeUrl = $this->config->oidc->appHost . "auth?" . http_build_query($codeParams);
        $this->requestlog->saveRequestLog($codeUrl, 'oidc' , 'getCode', 'GET', $codeParams);
        header('Location: ' . $codeUrl);
    }

    /**
     * Get access_token.
     *
     * @access public
     * @return void
     */
    public function getToken($code)
    {
        $this->app->loadConfig('oidc');
        $getTokenUrl = $this->config->oidc->appHost . "token";
        $tokenParams = $this->meageConfig($this->config->oidc->tokenParams);
        $tokenParams['code'] = $code;

        $tokenResponse = $this->requestlog->http($getTokenUrl, $tokenParams, 'POST');
        $this->requestlog->saveRequestLog($getTokenUrl, 'oidc' , 'gettoken', 'POST', '', $tokenResponse);
        return json_decode($tokenResponse, true);
    }

    /**
     * Get user.
     *
     * params  $token
     * @access public
     * @return string
     */
    public function getUserInfo($token)
    {
        $userInfoUrl = $this->config->oidc->appHost . "userinfo";
        $headers     = array("authorization: Bearer $token");
        $userInfo    = $this->requestlog->http($userInfoUrl, array(), 'POST', 'json', $headers);

        $this->requestlog->saveRequestLog($userInfoUrl, 'oidc' , 'userinfo', 'POST', '', $userInfo);
        $userInfo    = json_decode($userInfo, true);
        return $userInfo;
    }

    /**
     * checkUserExist
     *
     * @param mixed $userInfo
     * @access public
     * @return void
     */
    public function checkUserExist($userInfo)
    {
        $user = $this->dao->select('*')->from(TABLE_USER)->where('oidcAccount')->eq($userInfo['preferred_username'])->fetch();
        if($user)
        {
            $this->login($user);
        }
        else
        {
            $params = array('linkAccount' => base64_encode($userInfo['preferred_username']));
            $link = helper::createLink('user', 'oidcinitaccount', $params, 'html');
            $link = str_replace('oidc.php', 'index.php', $link);

            return print(js::locate($link, 'parent'));
        }
    }

    /**
     * login.
     *
     * @param mixed $user
     * @access public
     * @return void
     */
    public function login($user)
    {
        $url = $_SERVER['SERVER_NAME'] . helper::createLink('my', 'index');
        $url = str_replace('oidc', 'index', $url);

        /* Verify user identity. */
        $hasTmp = $this->loadModel('user')->checkTmp();
        if($hasTmp === false)
        {
            echo "<html><head><meta charset='utf-8'></head>";
            echo "<body><table align='center' style='width:700px; margin-top:100px; border:1px solid gray; font-size:14px;'><tr><td style='padding:8px'>";
            echo "<div style='margin-bottom:8px;'>不能创建临时目录，请确认目录<strong style='color:#ed980f'>{$app->tmpRoot}</strong>是否存在并有操作权限。</div>";
            echo "<div>Can't create tmp directory, make sure the directory <strong style='color:#ed980f'>{$app->tmpRoot}</strong> exists and has permission to operate.</div>";
            die("</td></tr></table></body></html>");
        }

        /* Successful login, jump to the home page. */
        $password = $user->password;
        $account  = $user->account;
        unset($_SESSION['rand']);
        $user = $this->user->identify($account, $password);
        $_POST['account'] = $account;
        $this->app->setVision();
        $this->user->login($user);

        echo '<script language="JavaScript">window.location.href="http://' . $url . '"</script>';
    }

    /**
     * User logout.
     *
     * @access public
     * @return string
     */
    public function logout()
    {
        if($this->app->config->ssoauth->status == 'disabled') return false;

        $systemUrl = common::getSysURL();
        $url       = $this->app->config->ssoauth->ssoLink;

        $query = array(
            'redirctToUrl'    => $systemUrl,
            'redirectToLogin' => 'true',
            'entityId'        => $this->app->config->ssoauth->clientID
        );
        $params = $query;
        $query  = http_build_query($query);
        $url    = $url . '/idp/profile/OAUTH2/Redirect/GLO?' . $query;

        $this->loadModel('requestlog')->http($url, $params, 'GET');
        $this->requestlog->saveRequestLog($url, 'sso', 'logout', 'GET', $params, '', 'success');
    }
}
