<?php
class user extends control
{
    /**
     * Init account from oidc.
     *
     * @param string $linkAccount
     * @access public
     * @return void
     */
    public function oidcInitAccount($oidcAccount = '')
    {
        if(empty($oidcAccount)) return print(js::locate($this->createLink('user', 'login'), 'parent'));

        $oidcAccount = base64_decode($oidcAccount);
        $user = $this->dao->select('*')->from(TABLE_USER)->where('oidcAccount')->eq($oidcAccount)->fetch();
        if($user) return print(js::error($this->lang->user->userHasExists) . js::locate($this->createLink('user', 'login'), 'parent'));

        /* Passed account and password by post or get. */
        if(!empty($_POST))
        {
            $account  = '';
            $password = '';
            if($this->post->account)  $account  = $this->post->account;
            if($this->post->password) $password = $this->post->password;
            $account = trim($account);

            if($_POST['isCreated'])
            {
                $password = substr($this->post->password1, 0, 32);
                $this->user->createUserFromOIDC($account, $password, $oidcAccount);
                $password = $this->session->rand ? md5($password. $this->session->rand) : $password;
            }

            if(dao::isError()) return print(js::error(dao::getError()));

            $this->checkLogin($account);
            $this->oidcLogin($account, $password, $oidcAccount);
        }
        else
        {
            setcookie('tab', false, time(), $this->config->webRoot);

            $this->loadModel('misc');
            $this->loadModel('extension');
            $this->view->noGDLib       = sprintf($this->lang->misc->noGDLib, common::getSysURL() . $this->config->webRoot, '', false, true);
            $this->view->title         = $this->lang->user->login;
            $this->view->s             = zget($this->config->global, 'sn', '');
            $this->view->keepLogin     = $this->cookie->keepLogin ? $this->cookie->keepLogin : 'off';
            $this->view->rand          = $this->user->updateSessionRandom();
            $this->view->unsafeSites   = $this->misc->checkOneClickPackage();
            $this->view->plugins       = $this->extension->getExpiringPlugins(true);
            $this->view->oidcAccount   = str_replace('@', '' ,$oidcAccount);
            $this->display();
        }
    }



    /**
     * Check login.
     *
     * @param mixed $account
     * @access public
     * @return void
     */
    public function checkLogin($account)
    {
        if($this->user->checkLocked($account))
        {
            $response['result']  = 'fail';
            $response['message'] = sprintf($this->lang->user->loginLocked, $this->config->user->lockMinutes);
            if($this->app->getViewType() == 'json') return print(helper::removeUTF8Bom(json_encode(array('status' => 'failed', 'reason' => $response['message']))));
            return $this->send($response);
        }

        if((!empty($this->config->safe->loginCaptcha) and strtolower($this->post->captcha) != strtolower($this->session->captcha) and $this->app->getViewType() != 'json'))
        {
            $response['result']  = 'fail';
            $response['message'] = $this->lang->user->errorCaptcha;
            return $this->send($response);
        }
    }


    /**
     * oidc login.
     *
     * @param mixed $account
     * @param mixed $password
     * @param mixed $oidcAccount
     * @access public
     * @return void
     */
    public function oidcLogin($account, $password, $oidcAccount)
    {
        $user = $this->user->identify($account, $password);

        if($user)
        {
            /* Set user group, rights, view and aword login score. */
            $user = $this->user->login($user);
            $this->dao->update(TABLE_USER)
                ->set('oidcAccount')->eq($oidcAccount)
                ->where('account')->eq($user->account)
                ->exec();

            return print(js::locate($this->createLink('my', 'index'), 'parent'));
        }
        else
        {
            $response['result']  = 'fail';
            $fails = $this->user->failPlus($account);

            $remainTimes = $this->config->user->failTimes - $fails;
            if($remainTimes <= 0)
            {
                $response['message'] = sprintf($this->lang->user->loginLocked, $this->config->user->lockMinutes);
                return $this->send($response);
            }
            else if($remainTimes <= 3)
            {
                $response['message'] = sprintf($this->lang->user->lockWarning, $remainTimes);
                return $this->send($response);
            }

            $response['message'] = $this->lang->user->linkFailed;
            if(dao::isError()) $response['message'] = dao::getError();
            return $this->send($response);
        }
    }
}
