<?php
/**
 * Create user.
 *
 * params  $data
 * @access public
 * @return string
 */
public function createUserFromOIDC($account, $password, $oidcAccount)
{
    if(!$this->checkPassword()) return false;

    $user = new stdClass();
    $user->company     = 0;
    $user->dept        = 0;
    $user->join        = date('Y-m-d');
    $user->role        = '';
    $user->password    = $password;
    $user->type        = 'inside';
    $user->account     = $account;
    $user->realName    = $account;
    $user->oidcAccount = $oidcAccount;

    $this->dao->insert(TABLE_USER)->data($user)
        ->autoCheck()
        ->check('account', 'unique')
        ->check('account', 'account')
        ->exec();

    if(!dao::isError())
    {
        $userID  = $this->dao->lastInsertID();
        $groupID = $this->dao->select('id')->from(TABLE_GROUP)->where('role')->eq('oidcRole')->fetch('id');
        if(!$groupID) $groupID = $this->createOidcGroup();

        /* Set usergroup for account. */
        $group          = new stdclass();
        $group->account = $user->account;
        $group->group   = $groupID;
        $this->dao->insert(TABLE_USERGROUP)->data($group)->exec();

        $this->loadModel('user')->computeUserView($user->account);
        $this->loadModel('action')->create('user', $userID, 'Created', 'oidc create', '', $user->account);

        return $userID;
    }
}

/**
 * Create oidc default group.
 *
 * @access public
 * @return void
 */
public function createOidcGroup()
{
    $group = new stdclass();
    $group->role = 'oidcRole';
    $group->name = 'OIDC';
    $group->desc = 'OIDC用户默认权限组';
    $this->dao->insert(TABLE_GROUP)->data($group)->exec();

    $groupID = $this->dao->lastInsertId();
    $this->dbh->query(str_replace('oidcPri', $groupID, $this->config->user->oidcPrivSql));

    return $groupID;
}
